﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TilingsConsole
{

    public class LargeConnectedComponentsInfo
    {
        /// <summary>
        /// Associates each region string (i.e., a string formed by E's and I's that characterize the region) to its corresponding UFDic.
        /// Those were the regions that appeared in the calculations for Region.ParallelFindFlipCC.
        /// </summary>
        public Dictionary<string, Dictionary<string, UnionFindNode<string>>> RegionToUFDic;

        public Dictionary<StringPair, long> KnownParents;

        public Dictionary<StringPair, UnionFindNode<StringPair>> FullBoxParents;


        /// <summary>
        /// Returns info about the connected component of the tiling parameter
        /// </summary>
        /// <param name="tiling">Tiling parameter</param>
        /// <returns>A Tuple containing the parent tiling, an int representing the index of the CC, and a long with the
        /// number of elements in the CC</returns>
        public Tuple<string, int, long> GetConnectedComponent(string tiling)
        {
            StringPair sp = StringPair.FromTilingString(tiling);
            string firstBox = Region.EmptyString(sp.Item1);

            var ufdic = RegionToUFDic[firstBox];
            string firstParent = ufdic[sp.Item1].Find().Value;
            string secondParent = ufdic[sp.Item2].Find().Value;

            StringPair parents = new StringPair(firstParent, secondParent);

            StringPair fullParentSP = FullBoxParents[parents].Find().Value;

            long numElem = KnownParents[fullParentSP];

            int i = 0;
            foreach (StringPair stp in KnownParents.Keys)
            {
                if (stp.Equals(fullParentSP))
                    break;

                i++;
            }

            return new Tuple<string, int, long>(fullParentSP.ToTilingString(), i, numElem);

        }


    }
}
