﻿using System;
using System.Collections.Generic;

namespace TilingsConsole
{
    public abstract class Move
    {
        public int InitPos;
        public DimerDirection InitialDirection;
        public DimerDirection FinalDirection;

        public Move(int initPos, DimerDirection initialDirection, DimerDirection finalDirection)
        {
            InitPos = initPos;
            InitialDirection = initialDirection;
            FinalDirection = finalDirection;
        }
    }

    // For the moment unused, but may come in handy in future implementations.
    public class Flip : Move
    {
        public Flip(int initPos, DimerDirection initialDirection, DimerDirection finalDirection)
            : base(initPos, initialDirection, finalDirection)
        {
        }
    }

    public class Trit : Move
    {
        public bool IsLowestCoordinateInaccessible;

        public Trit(int initPos, DimerDirection initialDirection, DimerDirection finalDirection, bool isLowestCoordinateInaccessible = false)
            :base(initPos,initialDirection,finalDirection)
        {
            IsLowestCoordinateInaccessible = isLowestCoordinateInaccessible;
        }
        /// <summary>
        /// Returns true iff the trit is positive. 
        /// </summary>
        /// <returns>A boolean (true iff the trit is positive)</returns>
        public bool IsPositive()
        {
            Dictionary<DimerDirection, int> directionsDir = new Dictionary<DimerDirection, int>();
            directionsDir[DimerDirection.Down] = 0;
            directionsDir[DimerDirection.Right] = 1;
            directionsDir[DimerDirection.Back] = 2;

            if (IsLowestCoordinateInaccessible)
            {
                return (directionsDir[FinalDirection] <= directionsDir[InitialDirection]);
            }

            int diffMod3 = (directionsDir[FinalDirection] - directionsDir[InitialDirection]) % 3;
            if (diffMod3 < 0)
                diffMod3 += 3;

            return diffMod3 != 1;
        }
    }
}