﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace TilingsConsole
{
    public class SVGDrawing
    {
        #region Drawing parameters
        List<string> Drawing = new List<string>();

        string preambule;
        List<string> BaseSquareList = new List<string>();
        List<string> DominoList = new List<string>();
        string ending = "</svg>";

        List<string> Miscellaneous = new List<string>();

        #endregion

        #region Parameters

        double squareSide = 30;

        System.Globalization.CultureInfo american = new System.Globalization.CultureInfo("en-US");

        #endregion

        #region Properties

        public string FirstTilingColor { set; get; }
        public string SecondTilingColor { set; get; }
        public string CommonTilingColor = "green"; //{ set; get; }

        #endregion

        #region Constructors
        public SVGDrawing()
        {


            // preambule = @"<svg xmlns=""http://www.w3.org/2000/svg"" version=""1.1"" height=""400%"">";

        }

        public SVGDrawing(string tiling, string filepath)
        {
            DrawTiling(tiling, filepath);
        }

        public SVGDrawing(string tilingPlus, string tilingMinus, string filepath)
        {
            DrawCycle(tilingPlus, tilingMinus, filepath);
        }

        #endregion

        #region Basic Helper Methods

        string BaseSquare(double posX, double posY)
        {


            string square = "";
            square += "<rect ";
            square += @"style=""fill:#e6e6e6;fill-rule:evenodd;stroke:#000000;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"" ";
            square += string.Format(american, @"id=""basesquare{0:F0}{1:F0}"" ", posX, posY);
            square += string.Format(american, @"width=""{0:F}"" ", squareSide);
            square += string.Format(american, @"height=""{0:F}"" ", squareSide);
            square += string.Format(american, @"x=""{0:F}"" ", posX);
            square += string.Format(american, @"y=""{0:F}"" /> ", posY);


            return square;
        }

        string EmptyBaseSquare(double posX, double posY)
        {

            string square = "";
            square += "<rect ";
            square += @"style=""fill:none;fill-rule:evenodd;stroke:#000000;stroke-width:1;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;stroke-miterlimit:4;stroke-dasharray:2, 2;stroke-dashoffset:0"" ";
            square += string.Format(american, @"id=""basesquare{0:F0}{1:F0}"" ", posX, posY);
            square += string.Format(american, @"width=""{0:F}"" ", squareSide);
            square += string.Format(american, @"height=""{0:F}"" ", squareSide);
            square += string.Format(american, @"x=""{0:F}"" ", posX);
            square += string.Format(american, @"y=""{0:F}"" /> ", posY);

            return square;

        }

        string Rectangle(double upperLeftX, double upperLeftY, double width, double height, string color)
        {

            StringBuilder rectangle = new StringBuilder("<rect ");
            rectangle.Append(string.Format(@"style=""fill:{0}"" ", color));

            rectangle.Append(string.Format(american, @"id=""rect{0:F0}{1:F0}"" ", upperLeftX, upperLeftY));
            rectangle.Append(string.Format(american, @"width=""{0}"" ", width));
            rectangle.Append(string.Format(american, @"height=""{0}"" ", height));
            rectangle.Append(string.Format(american, @"x=""{0}"" ", upperLeftX));
            rectangle.Append(string.Format(american, @"y=""{0}"" /> ", upperLeftY));

            return rectangle.ToString();

        }

        string Circle(double centerPosX, double centerPosY, double radius, string fillcolor, string strokecolor = "red")
        {
            return String.Format(american, @"<circle cx=""{0}"" cy=""{1}"" r=""{2}"" style=""fill:{3};stroke:{4};stroke-width:2px"" />", centerPosX, centerPosY, radius, fillcolor, strokecolor);
        }

        #endregion

        #region Specific Helper Methods

        void AddRectangleDomino(double startingSquarePosX, double startingSquarePosY, bool horizontal, string color = "red")
        {
            double width, height, upperLeftX, upperLeftY;

            if (horizontal)
            {
                width = 3 * squareSide / 2;
                height = squareSide / 2;
            }
            else
            {
                width = squareSide / 2;
                height = 3 * squareSide / 2;
            }

            upperLeftX = startingSquarePosX + squareSide / 4;
            upperLeftY = startingSquarePosY + squareSide / 4;


            DominoList.Add(Rectangle(upperLeftX, upperLeftY, width, height, color));
        }

        void AddCircleDomino(double startingSquareX, double startingSquareY, bool back, string frontcolor)
        {
            string color = (back ? "#ffffff" : frontcolor);
            double centerPosX = startingSquareX + squareSide / 2;
            double centerPosY = startingSquareY + squareSide / 2;
            double radius = squareSide / 4;

            DominoList.Add(Circle(centerPosX, centerPosY, radius, color, frontcolor));


        }

        public void DrawSingleTiling(string tiling, string color = "red", double initPosX = 100, double initPosY = 100)
        {
            int length = tiling.Length;
            double currentPosX = initPosX;
            double currentPosY = initPosY;


            for (int i = 0; i < length; i++)
            {
                char c = tiling[i];

                switch (c)
                {
                    case 'I':
                        BaseSquareList.Add(EmptyBaseSquare(currentPosX, currentPosY));
                        currentPosX += squareSide;      
                        continue;
                    case ',':
                        currentPosX = initPosX;
                        currentPosY += squareSide;
                        continue;
                    case ';':
                        //Review this completely!
                        initPosX = currentPosX + squareSide;
                        currentPosX = initPosX;
                        //currentPosY += 2 * squareSide;
                        currentPosY = initPosY;
                        continue;

                    case 'R':
                        AddRectangleDomino(currentPosX, currentPosY, true, color);
                        break;
                    case 'D':
                        AddRectangleDomino(currentPosX, currentPosY, false, color);
                        break;
                    case 'B':
                    case 'F':
                        AddCircleDomino(currentPosX, currentPosY, c == 'B', color);
                        break;
                    default:
                        break;
                }

                BaseSquareList.Add(BaseSquare(currentPosX, currentPosY));

                currentPosX += squareSide;
            }

            preambule = string.Format(@"<!--
    Drawing made by TilingsViewerSVG.
    Source tiling: {0}
-->
", tiling);
            preambule += string.Format(@"<svg xmlns=""http://www.w3.org/2000/svg"" version=""1.1"" width=""{0}"" height=""{1}"">", currentPosX, currentPosY + squareSide + 100);


        }

        void DrawTwoConcurrentDimers(double posX, double posY, char cPlus, char cMinus, string colorPlus, string colorMinus)
        {

            if ((cPlus == 'B') || (cPlus == 'F'))
            {
                string fillColor, strokeColor;
                if (cPlus == cMinus)
                {
                    fillColor = strokeColor = CommonTilingColor;

                    AddCircleCycleDomino(posX, posY, cPlus == 'B', fillColor);
                    //AddCircleDomino(posX, posY, cPlus == 'B', fillColor);
                }
                else
                {
                    fillColor = (cPlus == 'B' ? colorMinus : colorPlus);
                    strokeColor = (cMinus == 'B' ? colorMinus : colorPlus);

                    AddCircleCycleDomino(posX, posY, false, fillColor, strokeColor);
                    //AddCircleDomino(posX, posY, cPlus == 'B', fillColor);
                }


                return;

            }

            AddRectCycleDomino(posX, posY, cPlus == 'R', CommonTilingColor);
            //AddRectangleDomino(posX, posY, cPlus == 'R', CommonTilingColor);

        }

        void AddRectCycleDomino(double startingSquarePosX, double startingSquarePosY, bool horizontal, string color)
        {
            double width, height, upperLeftX, upperLeftY;

            if (horizontal)
            {
                width = 3 * squareSide / 4;
                height = squareSide / 4;

                upperLeftX = startingSquarePosX + 5 * squareSide / 8;
                upperLeftY = startingSquarePosY + 3 * squareSide / 8;
            }
            else
            {
                width = squareSide / 4;
                height = 3 * squareSide / 4;

                upperLeftX = startingSquarePosX + 3 * squareSide / 8;
                upperLeftY = startingSquarePosY + 5 * squareSide / 8;
            }

            DominoList.Add(Rectangle(upperLeftX, upperLeftY, width, height, color));

        }

        void AddCircleCycleDomino(double startingSquarePosX, double startingSquarePosY, bool back, string frontcolor, string strokeColor = null)
        {
            string fillColor = (back ? "white" : frontcolor);
            strokeColor = (strokeColor ?? frontcolor);
            double centerPosX = startingSquarePosX + squareSide / 2;
            double centerPosY = startingSquarePosY + squareSide / 2;
            double radius = squareSide / 8;

            DominoList.Add(Circle(centerPosX, centerPosY, radius, fillColor, strokeColor));



        }
        /// <summary>
        /// Convenient function for drawing cycles, given two tilings of the same region. Draws the cycles tilingPlus - tilingMinus
        /// </summary>
        /// <param name="tilingPlus">The first tiling that forms the cycle</param>
        /// <param name="tilingMinus">The second tiling - must be a tiling of the same region - in particular, the characters , ; and I must match</param>
        /// <param name="colorPlus">The color of the first tiling in the drawing</param>
        /// <param name="colorMinus"> The color of the second tiling in the drawing</param>
        /// <param name="initPosX">The X coordinate of the initial position of the drawing in the SVG file</param>
        /// <param name="initPosY">The Y coordinate of the initial position of the drawing in the SVG file</param>
        public void DrawCycleTilings(string tilingPlus, string tilingMinus, string colorPlus = "red", string colorMinus = "blue", double initPosX = 100, double initPosY = 100)
        {
            int length = tilingPlus.Length;
            if (tilingMinus.Length != length)
            {
                throw new ArgumentException("DrawCycleTilings: Length of the tilings do not match");
            }

            double currentPosX = initPosX, currentPosY = initPosY;

            for (int i = 0; i < length; i++)
            {
                char cPlus = tilingPlus[i], cMinus = tilingMinus[i];

                if ((cPlus == cMinus) || (cPlus == 'B' && cMinus == 'F') || (cPlus == 'F' && cMinus == 'B'))
                {
                    switch (cPlus)
                    {
                        case 'I':
                            currentPosX += squareSide;
                            continue;
                        case ',':
                            currentPosX = initPosX;
                            currentPosY += squareSide;
                            continue;
                        case ';':
                            //Review this completely!
                            initPosX = currentPosX + squareSide;
                            currentPosX = initPosX;
                            //currentPosY += 2 * squareSide;
                            currentPosY = initPosY;
                            continue;

                        case 'R':
                        case 'D':
                        case 'B':
                        case 'F':
                            DrawTwoConcurrentDimers(currentPosX, currentPosY, cPlus, cMinus, colorPlus, colorMinus);
                            break;
                    }
                }
                else
                {
                    switch (cPlus)
                    {
                        case 'R':
                            AddRectCycleDomino(currentPosX, currentPosY, true, colorPlus);
                            break;
                        case 'D':
                            AddRectCycleDomino(currentPosX, currentPosY, false, colorPlus);
                            break;
                        case 'B':
                        case 'F':
                            AddCircleCycleDomino(currentPosX, currentPosY, cPlus == 'B', colorPlus);
                            break;
                    }

                    switch (cMinus)
                    {
                        case 'R':
                            AddRectCycleDomino(currentPosX, currentPosY, true, colorMinus);
                            break;
                        case 'D':
                            AddRectCycleDomino(currentPosX, currentPosY, false, colorMinus);
                            break;
                        case 'B':
                        case 'F':
                            AddCircleCycleDomino(currentPosX, currentPosY, cMinus == 'B', colorMinus);
                            break;
                    }


                }

                BaseSquareList.Add(BaseSquare(currentPosX, currentPosY));

                currentPosX += squareSide;
            }

            preambule = string.Format(@"<!--
    Drawing made by TilingsViewerSVG.
    Source tiling 1: {0}
    Source tiling 2: {1}
-->", tilingPlus, tilingMinus);
            preambule += string.Format(@"<svg xmlns=""http://www.w3.org/2000/svg"" version=""1.1"" width=""{0}"" height=""{1}"">", currentPosX, currentPosY + squareSide + 100);
        }

        #endregion

        #region Drawing functions

        internal void Draw(string filepath)
        {
            Drawing.Add(preambule);
            Drawing.AddRange(BaseSquareList);
            Drawing.AddRange(DominoList);
            Drawing.AddRange(Miscellaneous);
            Drawing.Add(ending);

            FileWriter.WriteToFile(filepath, Drawing);

        }

        public void DrawTiling(string tiling, string filepath)
        {
            DrawSingleTiling(tiling, "red", 5, 5);

            Draw(filepath);
        }

        public static void NewDrawTiling(string tiling, string filepath)
        {
            new SVGDrawing(tiling, filepath);
        }

        public void DrawCycle(string tilingPlus, string tilingMinus, string filepath)
        {

            DrawCycleTilings(tilingPlus, tilingMinus);

            Draw(filepath);
        }

        public static void NewDrawCycle(string tilingPlus, string tilingMinus, string filepath)
        {
            new SVGDrawing(tilingPlus, tilingMinus, filepath);
        }

        //Throws exception if the cycle has more than two floors
        public static void NewDrawTwoFloorCycle(string tiling, string filePath)
        {
            var floors = tiling.Split(';');
            if (floors.Length != 2)
                throw new ArgumentOutOfRangeException("tiling", tiling, "Tiling must have two floors");

            string firstTiling = new string(floors[0].Select((x, i) => ((x == 'I') && (floors[1][i] != 'I') ? 'E' : x)).ToArray());
            string secondTiling = new string(floors[1].Select((x, i) => ((x == 'I') && (floors[0][i] != 'I') ? 'E' : ( x=='F' ? 'B' : x))).ToArray());

            NewDrawCycle(firstTiling, secondTiling, filePath);
        }



        #endregion

        #region Specific System Of Cycles Drawing

        string Vertex(double centerX, double centerY, double radius)
        {
            return Circle(centerX, centerY, radius, "black", "black");
        }

        string ArrowSegment(double firstCenterX, double firstCenterY, double secondCenterX, double secondCenterY, int strokeWidthInPx = 1)
        {
            double xDiff = secondCenterX - firstCenterX;
            double yDiff = secondCenterY - firstCenterY;
            double distance = Math.Round(Math.Sqrt(xDiff * xDiff + yDiff * yDiff), 2);
            double fractionOfDist = 0.25;

            double firstArrowXDiff = fractionOfDist * (xDiff - yDiff);
            double firstArrowYDiff = fractionOfDist * (xDiff + yDiff);

            double firstArrowCenterX = (firstCenterX + secondCenterX) / 2 - firstArrowXDiff;
            double firstArrowCenterY = (firstCenterY + secondCenterY) / 2 - firstArrowYDiff;

            return string.Format(american, @"<g style=""stroke:black,stroke-width:{4}px"">\r\n<path id=""seg{0:F0}{1:F0}{2:F0}{3:F0}"" d=""m {0},{1} {2},{3}"" />\r\n<path d=""m {5},{6} {7},{8} {9},{7}"" />\r\n</g>",
                firstArrowCenterX,firstArrowCenterY,secondCenterX,secondCenterY,strokeWidthInPx,firstArrowCenterX,firstArrowCenterY,firstArrowXDiff,firstArrowYDiff,-firstArrowYDiff);
        }

        void DrawSystemOfCycles(string tiling, double initPosX = 100, double initPosY = 100)
        {
            Miscellaneous.Add("<g>");

            double currentPosX = initPosX + squareSide/2, currentPosY = initPosY+squareSide/2;
            int i = 0;
            char c;

            while((c = tiling[i]) != ';')
            {
                i++;
            }
            int sqBelow = i + 1;
            i = 0;

            Dictionary<char, double> xDiffs = new Dictionary<char, double>(), yDiffs = new Dictionary<char,double>();
            xDiffs['R'] = yDiffs['D'] = squareSide;
            xDiffs['L'] = yDiffs['U'] = -squareSide;
            yDiffs['R'] = xDiffs['D'] = yDiffs['L'] = xDiffs['U'] = 0;
   
            
           
            while((c = tiling[i]) != ';')
            {
                if (c == tiling[i + sqBelow] || c == 'B')
                {
                    switch (c)
                    {
                        case 'I':
                            currentPosX += squareSide;
                            continue;
                        case ',':
                            currentPosX = initPosX;
                            currentPosY += squareSide;
                            continue;
                        case 'B':
                            Miscellaneous.Add(Vertex(currentPosX, currentPosY, squareSide / 10));
                            break;
                        default:
                            Miscellaneous.Add(Vertex(currentPosX, currentPosY, squareSide / 10));
                            Miscellaneous.Add(Vertex(currentPosX + xDiffs[c], currentPosY + yDiffs[c], squareSide / 10));
                            break;
                    }
                }
                else
                {
                    //CONTINUE!
                }
                i++;
            }

               


        }

        #endregion


    }

    class FileWriter
    {
        string filepath;


        public FileWriter(string path, string filename)
        {
            filepath = Path.Combine(path, filename);



        }
        [Obsolete]
        public void WriteToFile(List<string> stringList, bool append = false)
        {
            StreamWriter streamwriter = new StreamWriter(filepath, append);

            foreach (string s in stringList)
            {
                streamwriter.WriteLine(s);
            }

            streamwriter.Flush();

            streamwriter.Close();
        }

        public static void WriteToFile(string filepath, List<string> stringList, bool append = false)
        {

            using (StreamWriter streamwriter = new StreamWriter(filepath, append))
            {

                foreach (string s in stringList)
                {
                    streamwriter.WriteLine(s);
                }

                streamwriter.Flush();
            }



        }
    }
}
