﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TilingsConsole
{
   public class StringPair
    {
        public string Item1;
        public string Item2;

        public long NumberOfElements;

        public StringPair(string item1, string item2, long numElements = 0)
        {
            Item1 = item1;
            Item2 = item2;
            NumberOfElements = numElements;
        }

        public override string ToString()
        {
            return Item1 + " + " + Item2;
        }

        static public StringPair FromTilingString(string tiling)
        {
            Region r = Region.FromString(tiling);
            int halfDepth = r.Depth >> 1;

            Region upperRegion = new Region(new BoxDimensions(r.Width, r.Height, halfDepth));
            Region lowerRegion = new Region(new BoxDimensions(r.Width, r.Height, halfDepth));

            int halfLength = r.Length >> 1;


            for (int k = 0; k < upperRegion.Depth; k++)
                for (int j = 0; j < upperRegion.Height; j++)
                    for (int i = 0; i < upperRegion.Width; i++)
                    {
                        upperRegion[i, j, k] = r[i, j, halfDepth - k - 1];
                        lowerRegion[i, j, k] = r[i, j, halfDepth + k];

                        if (k == 0)
                        {
                            if (upperRegion[i, j, k] == DimerDirection.Back)
                                upperRegion[i, j, k] = DimerDirection.Inaccessible;

                            if (lowerRegion[i, j, k] == DimerDirection.Front)
                                lowerRegion[i, j, k] = DimerDirection.Inaccessible;
                        }

                        if (upperRegion[i, j, k] == DimerDirection.Front)
                            upperRegion[i, j, k] = DimerDirection.Back;
                        else if (upperRegion[i, j, k] == DimerDirection.Back)
                            upperRegion[i, j, k] = DimerDirection.Front;
                    }

            return new StringPair(lowerRegion.ToString(), upperRegion.ToString());
        }

        public string ToTilingString()
        {
            string[] separatedItem2 = Item2.Split(';');
            string upperTiling = string.Join(";", separatedItem2.Reverse());

            upperTiling = upperTiling.Replace('F', 'A');
            upperTiling = upperTiling.Replace('B', 'F');
            upperTiling = upperTiling.Replace('A', 'B');
            upperTiling = upperTiling.Replace('I', 'B');

            string lowerTiling = Item1.Replace('I', 'F');

            return upperTiling + ';' + lowerTiling;

        }

    }
}
